@@ -115,10 +115,7 @@ function( p,
   "c" <- ADoverload("c")
   "[<-" <- ADoverload("[<-")
 
-  # n_t2 = number of stanzas in a given multi-stanza group
-  # n_s2 = number of stanzas across multi-stanza groups
   n_s2 = stanza_data$n_s2
-  # n_g2 = number of multi-stanza groups (age-structured populations)
   n_g2 = stanza_data$n_g2
   Amat_g2 = stanza_data$stanzainfo_g2z[,'Amat']
   Wmatslope_g2 = stanza_data$stanzainfo_g2z[,'Wmatslope']
@@ -161,7 +158,7 @@ function( p,
   #Y_zz = matrix(nrow=0, ncol=4)
   Y_zz_g2 = NULL
   baseEggsStanza = baseSpawnBio = baseRzeroS = rep(0, n_g2)
-  leading_H_s2 = Q_s2 = Consumption_s2 = rep(0, n_s2)
+  leading_ratio_s2 = Q_s2 = Consumption_s2 = rep(0, n_s2)
   for( g2 in seq_len(n_g2) ){
     #
     Xg2_zz = X_zz_g2[[g2]]
@@ -175,13 +172,13 @@ function( p,
     which_leading = which(stanzainfo_t2z[,'lead']==1)
     leading_s2 = stanzainfo_t2z[which_leading,'s2']
 
-    # stanza for each age
+    #
     t2_a = Xg2_zz[,'t2']
 
     # WageS and QageS
     k = exp(p$log_K_g2[g2]) * 3     # 3 because W = L^3, i.e. converting VB length param K to weight param k
     WageS = (1 - exp(-k * (1 - d_g2[g2]) * AGE)) ^ inv1minus_d_g2[g2]
-    #WageS = exp(p$log_winf_z[g2]) * (1 - exp(-k * (1 - d_g2[g2]) * (AGE))) ^ inv1minus_d_g2[g2]
+    #WageS = exp(p$log_winf_z[1]) * (1 - exp(-k * (1 - d) * (AGE))) ^ (1 / (1 - d))
     QageS = WageS ^ d_g2[g2]
 
     # SurvS
@@ -201,26 +198,23 @@ function( p,
     baseSpawnBio[g2] = sum(NageS * fecundity_by_weight(WageS, p$Wmat_g2[g2], Wmatslope_g2[g2]) )
 
     #
-    leading_s = stanzainfo_t2z[which_leading,'s']
-    Consumption_s2[leading_s2] = exp(p$logQB_i[leading_s]) * exp(p$logB_i[leading_s])
+    s = stanzainfo_t2z[which_leading,'s']
+    Consumption_s2[leading_s2] = exp(p$logQB_i[s]) * exp(p$logB_i[s])
 
-    # Solve for consumptive demand Q_s2
+    # Solve for expected consumption
     for( t2 in seq_len(nrow(stanzainfo_t2z)) ){
       s = stanzainfo_t2z[t2,'s']
-      s2 = stanzainfo_t2z[t2,'s2']
       which_a = which( t2_a == stanzainfo_t2z[t2,'t2'] )
       p$logB_i[s] = log(sum(NageS[which_a] * WageS[which_a], na.rm=TRUE))
-      Q_s2[s2] = sum(NageS[which_a] * QageS[which_a], na.rm=TRUE)
+      Q_s2[stanzainfo_t2z[t2,'s2']] = sum(NageS[which_a] * QageS[which_a], na.rm=TRUE)
     }
 
     # Loop again for Cons and QB
-    # leading_H_s2 is Consumption_s2 / Q_s2 for leading stanza == H in the dW/dt = H W^d - K W for the leading-stanza for taxa_group s2
-    # H is constant across stanzas for a given multi-stanza group
     for( t2 in seq_len(nrow(stanzainfo_t2z)) ){
       s = stanzainfo_t2z[t2,'s']
       s2 = stanzainfo_t2z[t2,'s2']
-      leading_H_s2[s2] = Consumption_s2[leading_s2] / Q_s2[leading_s2]
-      Consumption_s2[s2] = Q_s2[s2] * leading_H_s2[s2]
+      leading_ratio_s2[s2] = Consumption_s2[leading_s2] / Q_s2[leading_s2]
+      Consumption_s2[s2] = Q_s2[s2] * leading_ratio_s2[s2]
       p$logQB_i[s] = log(Consumption_s2[s2]) - p$logB_i[s]
     }
 
@@ -231,17 +225,13 @@ function( p,
       s2 = stanzainfo_t2z[t2,'s2']
       which_a = which( t2_a == stanzainfo_t2z[t2,'t2'] )
       if(which_a[length(which_a)]==length(WageS)) which_a = which_a[-length(which_a)]
-      # W(a+1) = W(a) + (Food/Pred) * SplitAlpha - 3 * k_length / STEPS_PER_YEAR
       # W(a+1) = vbm_g2 * W(a) + (Food/Pred) * SplitAlpha
       # SplitAlpha = (W(a+1) - vbm_g2 * W(a)) / (Food/Pred)
-      SplitAlpha[which_a] = (WageS[which_a+1] - vbm_g2[g2] * WageS[which_a]) / leading_H_s2[s2]
+      SplitAlpha[which_a] = (WageS[which_a+1] - vbm_g2[g2] * WageS[which_a]) / leading_ratio_s2[s2]
       SplitAlpha[length(SplitAlpha)] = SplitAlpha[length(SplitAlpha)-1]
-      #
     }
 
     # Stack
-    # WageS, log_NageS, and QageS are equilibrium values and will be updated each time
-    # SplitAlpha is constant over time
     Yg2_zz = cbind( WageS=WageS, log_NageS=log(NageS), QageS=QageS, SplitAlpha=SplitAlpha)
     #Y_zz = rbind( Y_zz, Yg2_zz )  # deparse.level=0 avoids RTMB error
     Y_zz_g2[[g2]] = Yg2_zz
@@ -272,11 +262,13 @@ function( p,
   "[<-" <- ADoverload("[<-")
   # Globals
   vbm_g2 = (1 - 3 * stanza_data$stanzainfo_g2z[,'K'] / STEPS_PER_YEAR)       # 3*K = k, where k is from dW = H*W^d - k*W
+  #SB_g2 = exp(p$logPB_i)[stanza_data$stanzainfo_s2z[,'s']] # Get it to be class-advector
   SB_g2 = Eggs_g2 = rep(0, stanza_data$n_g2 )
   Wmat_g2 = p$Wmat_g2
   Amat_g2 = stanza_data$stanzainfo_g2z[,'Amat']
   Wmatslope_g2 = stanza_data$stanzainfo_g2z[,'Wmatslope']
   #Wmat_g2 = exp(p$log_winf_z[1]) * stanza_data$stanzainfo_g2z[,'Wmat']
+  #Y_zz = p$Y_zz
   Y_zz_g2 = p$Y_zz_g2
   #X_zz = stanza_data$X_zz
   X_zz_g2 = stanza_data$X_zz_g2
@@ -335,14 +327,14 @@ function( p,
     which_z = which( Xg2_zz[,'t2']==stanza_data$stanzainfo_s2z[s2,'t2'] )
 
     #
-    consumptive_demand = sum( exp(Yg2_zz[which_z,'log_NageS']) * Yg2_zz[which_z,'QageS'] )
+    stanzaPred = sum( exp(Yg2_zz[which_z,'log_NageS']) * Yg2_zz[which_z,'QageS'] )
     state_Biomass = sum( exp(Yg2_zz[which_z,'log_NageS']) * Yg2_zz[which_z,'WageS'] )
 
     # Copy ecosim.cpp#L890
     Z_s2[s2] = (LossPropToB_s[s] / state_Biomass) + F_s[s]
     #Su = exp(-Zrate / STEPS_PER_YEAR);
     log_Su = -Z_s2[s2] / STEPS_PER_YEAR
-    QB_s2[s2] = FoodGain_s[s] / consumptive_demand
+    QB_s2[s2] = FoodGain_s[s] / stanzaPred
 
     # Vectorized version
     #Yg2_zz[which_z,'NageS'] = Yg2_zz[which_z,'NageS'] * Su;
@@ -377,6 +369,7 @@ function( p,
     Y_zz_g2[[g2]] = Yg2_zz
   }
 
+
   # Update and return
   out = list(
     Y_zz_g2 = Y_zz_g2,
